==> ppdiffusers-0.14.2-py3-none-any.whl: tests/pipelines/dit/test_dit.py <==
# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
# Copyright 2023 The HuggingFace Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import gc
import unittest

import numpy as np
import paddle
from ppdiffusers_test.pipeline_params import (
    CLASS_CONDITIONED_IMAGE_GENERATION_BATCH_PARAMS,
    CLASS_CONDITIONED_IMAGE_GENERATION_PARAMS,
)
from ppdiffusers_test.test_pipelines_common import PipelineTesterMixin

from ppdiffusers import (
    AutoencoderKL,
    DDIMScheduler,
    DiTPipeline,
    DPMSolverMultistepScheduler,
    Transformer2DModel,
)
from ppdiffusers.utils import slow
from ppdiffusers.utils.testing_utils import require_paddle_gpu


class DiTPipelineFastTests(PipelineTesterMixin, unittest.TestCase):
    pipeline_class = DiTPipeline
    test_cpu_offload = False
    params = CLASS_CONDITIONED_IMAGE_GENERATION_PARAMS
    required_optional_params = PipelineTesterMixin.required_optional_params - {
        "latents",
        "num_images_per_prompt",
        "callback",
        "callback_steps",
    }
    batch_params = CLASS_CONDITIONED_IMAGE_GENERATION_BATCH_PARAMS

    def get_dummy_components(self):
        paddle.seed(0)
        transformer = Transformer2DModel(
            sample_size=16,
            num_layers=2,
            patch_size=4,
            attention_head_dim=8,
            num_attention_heads=2,
            in_channels=4,
            out_channels=8,
            attention_bias=True,
            activation_fn="gelu-approximate",
            num_embeds_ada_norm=1000,
            norm_type="ada_norm_zero",
            norm_elementwise_affine=False,
        )
        vae = AutoencoderKL()
        scheduler = DDIMScheduler()
        components = {"transformer": transformer.eval(), "vae": vae.eval(), "scheduler": scheduler}
        return components

    def get_dummy_inputs(self, seed=0):
        generator = paddle.Generator().manual_seed(seed)

        inputs = {"class_labels": [1], "generator": generator, "num_inference_steps": 2, "output_type": "numpy"}
        return inputs

    def test_inference(self):
        components = self.get_dummy_components()
        pipe = self.pipeline_class(**components)
        pipe.set_progress_bar_config(disable=None)
        inputs = self.get_dummy_inputs()
        image = pipe(**inputs).images
        image_slice = image[0, -3:, -3:, -1]
        self.assertEqual(image.shape, (1, 16, 16, 3))
        expected_slice = np.array([0.34939063, 0.0, 0.85213435, 1.0, 1.0, 0.35904014, 0.8031484, 0.0, 0.13307571])
        max_diff = np.abs(image_slice.flatten() - expected_slice).max()
        self.assertLessEqual(max_diff, 0.001)

    def test_inference_batch_single_identical(self):
        self._test_inference_batch_single_identical(relax_max_difference=True)


@require_paddle_gpu
@slow
class DiTPipelineIntegrationTests(unittest.TestCase):
    def tearDown(self):
        super().tearDown()
        gc.collect()
        paddle.device.cuda.empty_cache()

    def test_dit_256(self):
        generator = paddle.Generator().manual_seed(0)
        pipe = DiTPipeline.from_pretrained("facebook/DiT-XL-2-256")
        pipe.to("gpu")

        words = ["vase", "umbrella", "white shark", "white wolf"]
        ids = pipe.get_label_ids(words)
        images = pipe(ids, generator=generator, num_inference_steps=40, output_type="np").images
        expected_slices = np.array(
            [
                [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
                [0.0, 0.0, 0.0, 0.0, 0.0016301274299621582, 0.0, 0.0, 0.0, 0.0],
                [
                    0.434637188911438,
                    0.4323567748069763,
                    0.4406988322734833,
                    0.442973256111145,
                    0.4462621212005615,
                    0.45129328966140747,
                    0.41893237829208374,
                    0.42390328645706177,
                    0.3906112015247345,
                ],
                [
                    0.9986965656280518,
                    0.9948190450668335,
                    0.9841029644012451,
                    0.9911775588989258,
                    0.9871039390563965,
                    0.9874314069747925,
                    0.9822297096252441,
                    0.9997426271438599,
                    1.0,
                ],
            ]
        )

        for word, image, expected_slice in zip(words, images, expected_slices):
            # expected_image = load_numpy(
            #     f"https://huggingface.co/datasets/hf-internal-testing/diffusers-images/resolve/main/dit/{word}.npy"
            # )
            assert image.shape == (256, 256, 3)
            image_slice = image[-3:, -3:, -1]
            assert np.abs((image_slice.flatten() - expected_slice).max()) < 0.001

    def test_dit_512_fp16(self):
        pipe = DiTPipeline.from_pretrained("facebook/DiT-XL-2-512", paddle_dtype=paddle.float16)
        pipe.scheduler = DPMSolverMultistepScheduler.from_config(pipe.scheduler.config)
        pipe.to("gpu")

        words = ["vase", "umbrella"]
        ids = pipe.get_label_ids(words)
        generator = paddle.Generator().manual_seed(0)
        images = pipe(ids, generator=generator, num_inference_steps=25, output_type="np").images

        expected_slices = np.array(
            [
                [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.994140625],
                [
                    0.0,
                    0.0,
                    0.01708984375,
                    0.024658203125,
                    0.0830078125,
                    0.134521484375,
                    0.175537109375,
                    0.33740234375,
                    0.207763671875,
                ],
            ]
        )

        for word, image, expected_slice in zip(words, images, expected_slices):
            # expected_image = load_numpy(
            #     f"https://huggingface.co/datasets/hf-internal-testing/diffusers-images/resolve/main/dit/{word}_fp16.npy"
            # )
            assert image.shape == (512, 512, 3)
            image_slice = image[-3:, -3:, -1]
            # TODO make this pass, maybe cased by DPMSolverMultistepScheduler
            assert np.abs((image_slice.flatten() - expected_slice).max()) < 0.75

==> ppdiffusers-0.16.3-py3-none-any.whl: tests/pipelines/dit/test_dit.py <==
# Copyright (c) 2023 PaddlePaddle Authors. All Rights Reserved.
# Copyright 2023 The HuggingFace Team. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import gc
import unittest

import numpy as np
import paddle

from ppdiffusers import (
    AutoencoderKL,
    DDIMScheduler,
    DiTPipeline,
    DPMSolverMultistepScheduler,
    Transformer2DModel, )
from ppdiffusers.utils import slow
from ppdiffusers.utils.testing_utils import require_paddle_gpu

from ..pipeline_params import (
    CLASS_CONDITIONED_IMAGE_GENERATION_BATCH_PARAMS,
    CLASS_CONDITIONED_IMAGE_GENERATION_PARAMS, )
from ..test_pipelines_common import PipelineTesterMixin


class DiTPipelineFastTests(PipelineTesterMixin, unittest.TestCase):
    pipeline_class = DiTPipeline
    test_cpu_offload = False
    params = CLASS_CONDITIONED_IMAGE_GENERATION_PARAMS
    required_optional_params = PipelineTesterMixin.required_optional_params - {
        "latents",
        "num_images_per_prompt",
        "callback",
        "callback_steps",
    }
    batch_params = CLASS_CONDITIONED_IMAGE_GENERATION_BATCH_PARAMS

    def get_dummy_components(self):
        paddle.seed(0)
        transformer = Transformer2DModel(
            sample_size=16,
            num_layers=2,
            patch_size=4,
            attention_head_dim=8,
            num_attention_heads=2,
            in_channels=4,
            out_channels=8,
            attention_bias=True,
            activation_fn="gelu-approximate",
            num_embeds_ada_norm=1000,
            norm_type="ada_norm_zero",
            norm_elementwise_affine=False, )
        vae = AutoencoderKL()
        scheduler = DDIMScheduler()
        components = {
            "transformer": transformer.eval(),
            "vae": vae.eval(),
            "scheduler": scheduler
        }
        return components

    def get_dummy_inputs(self, seed=0):
        generator = paddle.Generator().manual_seed(seed)

        inputs = {
            "class_labels": [1],
            "generator": generator,
            "num_inference_steps": 2,
            "output_type": "numpy"
        }
        return inputs

    def test_inference(self):
        components = self.get_dummy_components()
        pipe = self.pipeline_class(**components)
        pipe.set_progress_bar_config(disable=None)
        inputs = self.get_dummy_inputs()
        image = pipe(**inputs).images
        image_slice = image[0, -3:, -3:, -1]
        self.assertEqual(image.shape, (1, 16, 16, 3))
        print(image_slice.flatten())
        expected_slice = np.array([
            0.28088313, 0.0, 0.8108508, 1.0, 1.0, 0.47994, 0.9075564, 0.0,
            0.14398015
        ])
        max_diff = np.abs(image_slice.flatten() - expected_slice).max()
        self.assertLessEqual(max_diff, 0.001)

    def test_inference_batch_single_identical(self):
        self._test_inference_batch_single_identical(
            relax_max_difference=True, expected_max_diff=1e-3)

    def test_xformers_attention_forwardGenerator_pass(self):
        self._test_xformers_attention_forwardGenerator_pass(
            expected_max_diff=1e-3)


@require_paddle_gpu
@slow
class DiTPipelineIntegrationTests(unittest.TestCase):
    def tearDown(self):
        super().tearDown()
        gc.collect()
        paddle.device.cuda.empty_cache()

    def test_dit_256(self):
        generator = paddle.Generator().manual_seed(0)
        pipe = DiTPipeline.from_pretrained("facebook/DiT-XL-2-256")
        pipe.to("gpu")

        words = ["vase", "umbrella", "white shark", "white wolf"]
        ids = pipe.get_label_ids(words)
        images = pipe(
            ids, generator=generator, num_inference_steps=40,
            output_type="np").images
        expected_slices = np.array([
            [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
            [0.0, 0.0, 0.0, 0.0, 0.0016301274299621582, 0.0, 0.0, 0.0, 0.0],
            [
                0.434637188911438,
                0.4323567748069763,
                0.4406988322734833,
                0.442973256111145,
                0.4462621212005615,
                0.45129328966140747,
                0.41893237829208374,
                0.42390328645706177,
                0.3906112015247345,
            ],
            [
                0.9986965656280518,
                0.9948190450668335,
                0.9841029644012451,
                0.9911775588989258,
                0.9871039390563965,
                0.9874314069747925,
                0.9822297096252441,
                0.9997426271438599,
                1.0,
            ],
        ])

        for word, image, expected_slice in zip(words, images, expected_slices):
            # expected_image = load_numpy(
            #     f"https://huggingface.co/datasets/hf-internal-testing/diffusers-images/resolve/main/dit/{word}.npy"
            # )
            assert image.shape == (256, 256, 3)
            image_slice = image[-3:, -3:, -1]
            assert np.abs((image_slice.flatten() - expected_slice).max(
            )) < 0.001

    def test_dit_512_fp16(self):
        pipe = DiTPipeline.from_pretrained(
            "facebook/DiT-XL-2-512", paddle_dtype=paddle.float16)
        pipe.scheduler = DPMSolverMultistepScheduler.from_config(
            pipe.scheduler.config)
        pipe.to("gpu")

        words = ["vase", "umbrella"]
        ids = pipe.get_label_ids(words)
        generator = paddle.Generator().manual_seed(0)
        images = pipe(
            ids, generator=generator, num_inference_steps=25,
            output_type="np").images

        expected_slices = np.array([
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 0.994140625],
            [
                0.0,
                0.0,
                0.01708984375,
                0.024658203125,
                0.0830078125,
                0.134521484375,
                0.175537109375,
                0.33740234375,
                0.207763671875,
            ],
        ])

        for word, image, expected_slice in zip(words, images, expected_slices):
            # expected_image = load_numpy(
            #     f"https://huggingface.co/datasets/hf-internal-testing/diffusers-images/resolve/main/dit/{word}_fp16.npy"
            # )
            assert image.shape == (512, 512, 3)
            image_slice = image[-3:, -3:, -1]
            # TODO make this pass, maybe cased by DPMSolverMultistepScheduler
            assert np.abs((image_slice.flatten() - expected_slice).max()) < 0.75
